#!/bin/bash

#
#  Copyright (C) 2017-2019, Thomas Maier-Komor
#
#  This source file belongs to Wire-Format-Compiler.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

REPOSITORY=ssh://strahd/user/git/wfc.git

echo preparing clean build

./configure || exit
make distclean
make || exit

RELEASE=`bin/wfc -V | head -1 | sed 's/.*R/R/;s/ (hg.*//'`
hg tags | grep $RELEASE && echo release tag already exists && exit

difflines=`hg diff | wc -l`
if [ "$difflines" != "0" ]; then
	echo "uncommitted changes"
	exit
fi

echo preparing release $RELEASE - ready to start?
read

if [ -d wfc ]; then
	echo remove existing release directory?
	read
	rm -rf wfc
fi

git clone $REPOSITORY wfc
rm -r wfc/*
hg archive wfc
cd wfc
rm -r `cat privatefiles`
git add .
git ls-files --deleted | xargs git rm

echo done preparing git commit - ready to commit?
read
git commit -m "release $RELEASE" || exit

echo starting build tests
./configure || exit
make || exit
make mingw || exit
make mingw64 || exit
cd testsuite
./runtests.sh || exit
cd ..

echo ready to push?
read
git push

cd ..
hg tag $RELEASE

echo ready to push release tag to development repository?
read
hg push
