//#include "avrp1.h" - include via command line
#include <avr/io.h>
#include <avr/pgmspace.h>
#define F_CPU 1000000
#include <util/delay.h>

uint16_t readCoreTemperature()
{
	ADMUX |= (1<<REFS1)|(1<<REFS0);	// select 1.1V reference
	ADMUX |= (1<<MUX3);
	ADMUX &= ~((1<<MUX2)|(1<<MUX1)|(1<<MUX0));
	ADCSRA |= (1<<ADEN);	// enable ADC
	ADCSRA |= (1<<ADSC);	// start conversion
	while ((ADCSRA & (1<<ADIF)) == 0);	// wait for conversion to complete
	uint16_t r;
	r = ADCH << 8;
	r |= ADCL;
	return r;
}

void uart_write(uint8_t d)
{
	while (!UCSR0A & (1<<UDRE0));
	UDR0 = d;
}


#ifdef VIA_BUFFER
#define send send_via_mem
void uart_send(const uint8_t *data, size_t n)
{
	while (n--) {
		while (!UCSR0A & (1<<UDRE0));
		UDR0 = *data++;
	}
}


void send_via_mem(EmbeddedSystem *es)
{
	uint8_t buf[128];
	assert(es->calcSize() < sizeof(buf));
	size_t n = es->toMemory(buf,sizeof(buf));
	uart_send(buf,n);
}

#else
#define send send_direct

void send_direct(EmbeddedSystem *es)
{
#ifdef PUTARG
	es->toWire(uart_write);
#else
	es->toWire();
#endif
}
#endif


int main()
{
	EmbeddedSystem es;
	es.set_name(PSTR("balkon-sensor"));
	es.set_system(PSTR("avrp1"));
	es.set_revision(PSTR("V1.0"));
	Sensor_AD_16Bit *a0 = es.add_ad_16bit();
	a0->set_name(PSTR("core temperature"));
	while (1) {
		a0->set_value(readCoreTemperature());
		send(&es);
		_delay_ms(1000);
	}
}
