#include "wfccore.h"
#include <assert.h>
#include <unistd.h> // for _POSIX_TIMERS

#if defined __x86 || defined __x86_64
inline uint64_t rdtsc()
{
	uint64_t msr;
	asm volatile (
		"cpuid\n\t"
		"rdtsc\n\t"    // Returns the time in EDX:EAX.
		"shl $32, %%rdx\n\t"  // Shift the upper bits left.
		"or %%rdx, %0"        // 'Or' in the lower bits.
		: "=a" (msr)
		:
		: "rdx");
	return msr;
}
/*
	uint32_t lower = 0, upper = 0;
	asm volatile ("cpuid\nrdtsc": "=a" (lower), "=d" (upper));
	uint64_t ull = ((uint64_t) upper << 32) + lower;
	return ull;
*/
#elif defined _POSIX_TIMERS && (_POSIX_TIMERS > 0)
#include <time.h>
inline uint64_t rdtsc()
{
	struct timespec t;
	clock_gettime(CLOCK_REALTIME,&t);
	uint64_t r = t.tv_nsec/1000000 + t.tv_sec*1000;
	return r;
}
#else
#define rdtsc() 0
#endif

uint64_t perftest_v2w_w2v()
{
	uint64_t t = rdtsc();
	for (uint64_t i = 0; i < 0x1000000000ULL; i += 1117) {
		uint8_t buf[16];
		varint_t a;
		int n = write_varint(buf,sizeof(buf),i);
		assert(n > 0);
		unsigned u = read_varint(buf,n,&a);
		assert(u == (unsigned)n);
		assert(a == i);
	}
	printf("perftest_v2w_w2v: %ld\n",rdtsc()-t);
	return 0;
}


int main()
{
	perftest_v2w_w2v();
	return 0;
}
