#include "sysconfig.h"

int read_varint(const uint8_t *,ssize_t,varint_t *v);

/* wfc-template:
 * function: skip_field
 * errorhandling: cancel
 */

ssize_t skip_field_default(const uint8_t *wire, ssize_t wl)
{
	if (wl < 2)
		$handle_error;
	ssize_t n = 0;
	while (wire[n++] & 0x80) {	// also skip field tags >4bit
		if (wl == n)
			$handle_error;
	}
	switch (wire[0]&0x7) {
	case 0:
		while (wire[n++]&0x80) {
			if (wl == n)
				$handle_error;
		}
		break;
	case 1:
		n += 8;
		break;
	case 2: {
			varint_t v;
			unsigned l = read_varint(wire+n,wl,&v);
			if (0 == l)
				$handle_error;
			n += v;
			n += l;
			break;
		}
	case 3:
		++n;
		break;
	case 4:
		n += 2;
		break;
	case 5:
		n += 4;
		break;
	default:
		$handle_error;
	}
	if (n > wl)
		$handle_error;
	return n;
}


/* wfc-template:
 * function: skip_field
 * errorhandling: cancel
 */
unsigned skip_field_0(const uint8_t *wire, ssize_t wl)
{
	if (wl < 2)
		$handle_error;
	unsigned n = 0;
	while (wire[n++] & 0x80) {	// also skip field tags >4bit
		if (wl == n)
			$handle_error;
	}
	switch (wire[0]&0x7) {
	case 0:
		while (wire[n++]&0x80) {
			if (wl == n)
				$handle_error;
		}
		break;
	case 1:
		n += 8;
		break;
	case 2: {
			varint_t v;
			unsigned l = read_varint(wire+n,wl,&v);
			if (0 == l)
				$handle_error;
			n += v;
			n += l;
			break;
		}
	case 3:
		++n;
		break;
	case 4:
		n += 2;
		break;
	case 5:
		n += 4;
		break;
	default:
		n = 0;
	}
	if (n > wl)
		$handle_error;
	return n;
}


/* wfc-template:
 * function: skip_field
 * errorhandling: cancel
 * description: works good for clang compiler
 */
unsigned skip_field_goto(const uint8_t *wire, ssize_t wl)
{
	unsigned n = 0;
	if (wl < 2)
		goto return_0;
	while (wire[n++] & 0x80) {	// also skip field tags >4bit
		if (wl == n)
			goto return_0;
	}
	switch (wire[0]&0x7) {
	case 0:
		while (wire[n++]&0x80) {
			if (wl == n)
				goto return_0;
		}
		break;
	case 1:
		n += 8;
		break;
	case 2: {
			varint_t v;
			unsigned l = read_varint(wire+n,wl,&v);
			if (0 == l)
				goto return_0;
			n += v;
			n += l;
			break;
		}
	case 3:
		++n;
		break;
	case 4:
		n += 2;
		break;
	case 5:
		n += 4;
		break;
	default:
		goto return_0;
	}
	if (n < wl)
		return n;
	return_0:
	$handle_error;
}



/* wfc-template:
 * function: skip_field
 * errorhandling: cancel
 */

unsigned skip_field_olddefault(const uint8_t *wire, ssize_t wl)
{
	if (wl < 2)
		$handle_error;
	unsigned n = 0;
	--wl;	// for byte 0 of tag
	while (wire[n++] & 0x80) {	// also skip field tags >4bit
		if (--wl == 0)
			$handle_error;
	}
	switch (wire[0]&0x7) {
	case 0:
		while (wire[n++]&0x80) {
			if (--wl == 0)
				$handle_error;
		}
		return n;
	case 1:
		if (wl < 8)
			$handle_error;
		n += 8;
		break;
	case 2: {
			varint_t l;
			unsigned v = read_varint(wire+n,wl,&l);
			if (0 == v)
				$handle_error;
			n += v;
			n += l;
			break;
		}
	case 3:
		if (wl < 1)
			$handle_error;
		++n;
		break;
	case 4:
		if (wl < 2)
			$handle_error;
		n += 2;
		break;
	case 5:
		if (wl < 4)
			$handle_error;
		n += 4;
		break;
	default:
		$handle_error;
	}
	return n;
}


/* wfc-template:
 * function: skip_field
 * errorhandling: cancel
 * syntax: gprotov3
 */

unsigned skip_field_gprotov3(const uint8_t *wire, ssize_t wl)
{
	if (wl < 2)
		$handle_error;
	unsigned n = 0;
	while (wire[n++] & 0x80) {	// also skip field tags >4bit
		if (wl == n)
			$handle_error;
	}
	switch (wire[0]&0x7) {
	case 0:
		while (wire[n++]&0x80) {
			if (wl == n)
				$handle_error;
		}
		break;
	case 1:
		n += 8;
		break;
	case 2: {
			varint_t v;
			unsigned l = read_varint(wire+n,wl,&v);
			if (0 == l)
				$handle_error;
			n += v;
			n += l;
			break;
		}
	case 5:
		n += 4;
		break;
	default:
		$handle_error;
	}
	if (n > wl)
		$handle_error;
	return n;
}


/* wfc-template:
 * function: skip_field
 * errorhandling: cancel
 */

unsigned skip_field_alternate1(const uint8_t *wire, ssize_t wl)
{
	if (wl < 2)
		$handle_error;
	unsigned n = 0;
	--wl;	// for byte 0 of tag
	while (wire[n++] & 0x80) {	// also skip
		if (--wl == 0)
			$handle_error;
	}
	switch (wire[0]&0x7) {
	case 0:
		while (wire[n++]&0x80) {
			if (--wl == 0)
				$handle_error;
		}
		return n;
	case 1:
		if (wl < 8)
			$handle_error;
		return n + 8;
	case 2: {
			varint_t l;
			if (0 == read_varint(wire+n,wl,&l))
				$handle_error;
			n += l;
			if (n > wl)
				$handle_error;
			return n;
		}
	case 3:
		if (wl < 1)
			$handle_error;
		return n + 1;
	case 4:
		if (wl < 2)
			$handle_error;
		return n + 2;
	case 5:
		if (wl < 4)
			$handle_error;
		return n + 4;
	default:
		$handle_error;
	}
}


/* wfc-template:
 * function: skip_field
 * errorhandling: none
 */

unsigned skip_field_nocheck(const uint8_t *wire, ssize_t wl)
{
	unsigned n = 0;
	while (wire[n++] & 0x80);
	switch (wire[0]&0x7) {
	case 0:
		while (wire[n++]&0x80);
		return n;
	case 1:
		return n + 8;
	case 2: {
			varint_t l;
			read_varint(wire+n,wl,&l);
			return n + l;
		}
	case 3:
		return n + 1;
	case 4:
		return n + 2;
	case 5:
		return n + 4;
	default:
		$handle_error;
	}
}


/* no-wfc-template:
 * function: skip_field
 * errorhandling: assert
 * buggy: true
 */

unsigned skip_field_assert(const uint8_t *wire, ssize_t wl)
{
	assert(wl > 1);
	switch (*wire&0x7) {
	case 0: {
			unsigned n = 1;
			while (--wl && (*++wire&0x80)) ++n;
			return n;
		}
	case 1:
		assert(wl >= 9);
		return 9;
	case 2: {
			varint_t l = 0;
			unsigned n = read_varint(++wire,wl-1,&l);
			assert(n != 0);
			l += n + 1;
			if (l < wl)
				return l;
			return wl;
		}
	case 3:
		assert(wl >= 2);
		return 2;
	case 4:
		assert(wl >= 3);
		return 3;
	case 5:
		assert(wl >= 5);
		return 5;
	default:
		assert(0);
	}
}


/* wfc-template:
 * function: skip_field
 * errorhandling: throw
 */

unsigned skip_field_throw(const uint8_t *wire, ssize_t wl)
{
	if (wl < 2)
		$handle_error;
	// buggy: tags > 4bit unsupported
	switch (*wire&0x7) {
	case 0: {
			unsigned n = 1;
			while (--wl && (*++wire&0x80)) ++n;
			return n;
		}
	case 1:
		return 9;
	case 2: {
			varint_t l = 0;
			unsigned n = read_varint(++wire,wl-1,&l);
			if (n == 0)
				return wl;
			l += n + 1;
			if ((ssize_t)l < wl)
				return l;
			return wl;
		}
	case 3:
		return 2;
	case 4:
		return 3;
	case 5:
		return 5;
	case 6:
		$handle_error;
	case 7:
		$handle_error;
	default:
		$handle_error;
	}
}


/* wfc-template:
 * function: skip_field
 * errorhandling: cancel
 */

unsigned skip_field_cancel2(const uint8_t *wire, ssize_t wl)
{
	if (wl < 2)
		$handle_error;
	// buggy: tags > 4bit unsupported
	switch (*wire&0x7) {
	case 0: {
			unsigned n = 1;
			while (--wl && (*++wire&0x80)) ++n;
			return n;
		}
	case 1:
		return 9;
	case 2: {
			varint_t l = 0;
			unsigned n = read_varint(++wire,wl-1,&l);
			if (n == 0)
				$handle_error;
			l += n + 1;
			if (l < wl)
				return l;
			$handle_error;
		}
	case 3:
		return 2;
	case 4:
		return 3;
	case 5:
		return 5;
	default:
		$handle_error;
	}
}


#ifdef TEST
#include <assert.h>

typedef uint64_t varint_t;

int read_varint(const uint8_t *wire, ssize_t wl, varint_t *r)
{
	uint8_t u8;
	int n = 0;
	*r = 0;
	do {
		if (--wl < 0)
			$handle_error;
		u8 = *wire++;
		*r |= (varint_t)(u8&~0x80) << (n*7);
		++n;
	} while ((u8 & 0x80) && (n < 10));
	if ((n == 10) && (u8 & 0x80))
		$handle_error;
	return n;
}

uint8_t err0[] = { 0x80 };
uint8_t err1[] = { 0x00 };
uint8_t err2[] = { 0x03 };
uint8_t err3[] = { 0x00, 0x80 };
uint8_t err4[] = { 0x04, 0x00 };
uint8_t err5[] = { 0x05, 0x00 };
uint8_t err6[] = { 0x11, 0x08 };
uint8_t err7[] = { 0x12, 0x80 };
uint8_t tag1byte[] = { 0x13, 0x01 };
uint8_t tag64byte[] = { 0x83, 0x01a, 0x00 };
uint8_t tag2short[] = { 0x24, 0x01a, 0x00 };
uint8_t tag3long[] = { 0x31, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
uint8_t tag1vint[] = { 0x10, 0x81, 0x01 };

void runtest(unsigned (*func)(const uint8_t *, ssize_t ))
{
	assert(0 == func(err0,sizeof(err0)));
	assert(0 == func(err1,sizeof(err1)));
	assert(0 == func(err2,sizeof(err2)));
	assert(0 == func(err3,sizeof(err3)));
	assert(0 == func(err4,sizeof(err4)));
	assert(0 == func(err5,sizeof(err5)));
	assert(0 == func(err6,sizeof(err6)));
	assert(0 == func(err7,sizeof(err7)));
	assert(2 == func(tag1byte,sizeof(tag1byte)));
	assert(3 == func(tag64byte,sizeof(tag64byte)));
	assert(3 == func(tag2short,sizeof(tag2short)));
	assert(9 == func(tag3long,sizeof(tag3long)));
	assert(3 == func(tag1vint,sizeof(tag1vint)));

}

int main()
{
	runtest(skip_field_default);
	//runtest(skip_field_assert);
}

#endif
